@@ -1,12 +1,11 @@
 # -*- coding: utf-8; mode: python; indent-tabs-mode: t; tab-width:4 -*-
-import sys, time, math, importlib, os, platform, os.path, configparser, csv
-import logging
+import sys, time, math, importlib, os, platform, os.path, configparser, csv, serial, fcntl
 from datetime import datetime
 from utils import cnf
 from language import languages
 from server import ScreenShotThread
-from PyQt6.QtCore import pyqtSignal, QObject, pyqtSlot, Qt, QTimer
-from PyQt6.QtGui import QIcon
+from PyQt5.QtCore import pyqtSignal, QObject, pyqtSlot, Qt, QTimer
+from PyQt5.QtGui import QIcon
 from QtVersion import *
 showVersions()
 
@@ -66,12 +65,8 @@ class PQG_ImageExporter(Exporter):
 
     def export(self, fileName=None, toBytes=False, copy=False):
         if fileName is None and not toBytes and not copy:
-            if USE_PYSIDE:
-                filter = ["*."+str(f)
-                          for f in QtGui.QImageWriter.supportedImageFormats()]
-            else:
-                filter = ["*."+bytes(f).decode('utf-8')
-                          for f in QtGui.QImageWriter.supportedImageFormats()]
+            filter = ["*."+bytes(f).decode('utf-8')
+                      for f in QtGui.QImageWriter.supportedImageFormats()]
             preferred = ['*.png', '*.tif', '*.jpg']
             for p in preferred[::-1]:
                 if p in filter:
@@ -249,7 +244,8 @@ mechanicsExpts = [
 otherExpts = [ 
 [QT_TRANSLATE_NOOP('MainWindow','Temperatue, PT100 Sensor'), ('7.1','pt100')],
 [QT_TRANSLATE_NOOP('MainWindow','Data Logger'), ('7.2','logger')],
-[QT_TRANSLATE_NOOP('MainWindow','Advanced Data Logger'), ('7.3','advanced_logger')]
+[QT_TRANSLATE_NOOP('MainWindow','Advanced Data Logger'), ('7.3','advanced_logger')],
+[QT_TRANSLATE_NOOP('MainWindow','Visual Programming Editor'), ('7.4','blockcoding')]
 ]
 
 modulesI2C = [ 
@@ -282,7 +278,7 @@ pythonCodes = [
 
 #---------------------------------------------------------------------
 		
-class helpWin(QWebEngineView, QScreen):
+class helpWin(QWebView):
 		
 	def closeEvent(self, e):
 		"""
@@ -305,8 +301,7 @@ class helpWin(QWebEngineView, QScreen):
 		:param lang: the desired language
 		"""
 
-		QWebEngineView.__init__(self)
-		QScreen.__init__(self)
+		QWebView.__init__(self)
 
 		self.parent=parent
 		self.lang=lang
@@ -318,10 +313,10 @@ class helpWin(QWebEngineView, QScreen):
 		self.setWindowTitle(self.tr('Help: %s') %name[0])
 		#self.setMaximumSize(QSize(500, 1200))
 		self.show()
-		screen = self.geometry()
+		screen = QDesktopWidget().screenGeometry()
 		self.move(screen.width()-self.width()-20, screen.height()-self.height()-60)
 
-class MainWindow(QMainWindow,QScreen):
+class MainWindow(QMainWindow):
 	WIDTH = 950
 	HEIGHT = 600
 	expWidget = None
@@ -350,7 +345,6 @@ class MainWindow(QMainWindow,QScreen):
 		:type  tr_qt: QTranslator
 		"""
 		QMainWindow.__init__(self)
-		QScreen.__init__(self)
 		self.lang=None # this will be set later, after self.translate() tries
 		self.app=app
 		self.tr_eyes=tr_eyes
@@ -380,11 +374,24 @@ class MainWindow(QMainWindow,QScreen):
 		self.shortcutActions={}
 		self.shortcuts={"Ctrl+R":self.reconnect,'Alt+s':self.screenshot,'Alt+p':self.screenshotPlot}
 		for a in self.shortcuts:
-			shortcut = QtGui.QShortcut(QtGui.QKeySequence(a), self)
+			shortcut = QtWidgets.QShortcut(QtGui.QKeySequence(a), self)
 			shortcut.activated.connect(self.shortcuts[a])
 			self.shortcutActions[a] = shortcut
 		self.screenShotThread = ScreenShotThread(self, port = 45594)
 		self.screenShotThread.start()
+
+		self.shortlist=self.getFreePorts(None)
+		self.deviceSelector = self.portSelectionDialog()
+		self.deviceSelector.setList(self.shortlist,p.H)
+
+
+		self.startTime = time.time()
+		self.timer = QtCore.QTimer()
+		self.timer.timeout.connect(self.locateDevices)
+		self.timer.start(500)
+
+
+
 		return
 
 	def closeEvent(self, event):
@@ -407,9 +414,8 @@ class MainWindow(QMainWindow,QScreen):
 		self.resize(self.WIDTH,self.HEIGHT)
 		self._x = 100
 		self._y = 10
-		palette = QPalette()
-		# background color
-		palette.setColor(QPalette.ColorRole(10), QColor(81,188,185)) #("#99ccff")) "#88bbcc"
+		palette = QPalette()								# background color
+		palette.setColor(QPalette.Background, QColor(81,188,185)) #("#99ccff")) "#88bbcc"
 		self.setPalette(palette)	
 
 		self.helpCB = QCheckBox(self.tr('Enable PopUp Help Window'))
@@ -421,7 +427,7 @@ class MainWindow(QMainWindow,QScreen):
 		self.statusBar.addWidget(self.helpCB)
 		
 		self.callExpt(electronicsExptsScope[0])					# Start the scope by default
-		self.screen = self.geometry()
+		self.screen = QDesktopWidget().screenGeometry()
 		self.show()
 		self.move(20, 20)
 		
@@ -452,6 +458,155 @@ class MainWindow(QMainWindow,QScreen):
 			self.sigConfig.emit(value)
 
 
+
+	def listPorts(self):
+		'''
+		Make a list of available serial ports. For auto scanning and connecting
+		'''
+		import glob
+		system_name = platform.system()
+		if system_name == "Windows":
+			# Scan for available ports.
+			available = []
+			for i in range(256):
+				try:
+					s = serial.Serial('COM%d'%i)
+					available.append('COM%d'%i)
+					s.close()
+				except serial.SerialException:
+					pass
+			return available
+		elif system_name == "Darwin":
+			# Mac
+			return glob.glob('/dev/tty*') + glob.glob('/dev/cu*')
+		else:
+			# Assume Linux or something else
+			return glob.glob('/dev/ttyACM*')
+
+	def isPortFree(self,portname):
+		try:
+			fd = serial.Serial(portname, 500000, stopbits=1, timeout = 1.0)
+			if fd.isOpen():
+				if 'inux' in platform.system(): #Linux based system
+					try:
+						fcntl.flock(fd.fileno(), fcntl.LOCK_EX | fcntl.LOCK_NB)
+						fd.close()
+						return True #Port is available
+					except IOError:
+						fd.close()
+						return False #Port is not available
+
+				else:
+					fd.close()
+					return True #Port is available
+			else:
+				fd.close()
+				return False #Port is not available
+
+		except serial.SerialException as ex:
+			return False #Port is not available
+
+	def getFreePorts(self,openPort=None):
+		'''
+		Find out which ports are currently free 
+		'''
+		portlist={}
+		for a in self.listPorts():
+			if a != openPort:
+				portlist[a] = self.isPortFree(a)
+			else:
+				portlist[a] = False
+		return portlist
+
+
+	def locateDevices(self):
+		L={}
+		try:L = self.getFreePorts(p.H.portname)
+		except Exception as e:pass
+		total = len(L)
+		menuChanged = False
+		if L != self.shortlist:
+			menuChanged = True
+
+			self.deviceSelector.setList(L,p.H)
+			#Check for, and handle disconnect event
+			if p.connected:
+				if platform.system() == "Windows": return #Ignore disconnect event on windows
+				if p.H.portname not in L:
+						self.setWindowTitle('Error : Device Disconnected')
+						QtWidgets.QMessageBox.warning(self, 'Connection Error', 'Device Disconnected. Please check the connections')
+						try:
+							p.H.portname = None
+							p.fd.close()
+						except:pass
+						p.connected = False
+
+			elif True in L.values():
+				reply = QtWidgets.QMessageBox.question(self, 'Connection', 'Device Available. Connect?', QtWidgets.QMessageBox.Yes, QtWidgets.QMessageBox.No)
+				if reply == QtWidgets.QMessageBox.Yes:
+					self.reconnect()
+
+			#update the shortlist
+			self.shortlist=L
+
+		
+	####################
+
+	def selectDevice(self):
+		if self.deviceSelector.exec_():
+			self.reconnect(port = self.deviceSelector.getSelection())
+
+
+	class portSelectionDialog(QtWidgets.QDialog):
+		def __init__(self,parent=None):
+			super(MainWindow.portSelectionDialog, self).__init__(parent)
+			self.button_layout = QtWidgets.QVBoxLayout()
+			self.setLayout(self.button_layout)
+			self.btns=[]
+			self.doneButton = QtWidgets.QPushButton("Done")
+			self.button_layout.addWidget(self.doneButton)
+			self.doneButton.clicked.connect(self.finished)
+			
+
+		def setList(self,L,handler):
+			for a in self.btns:
+				a.setParent(None)
+				del a
+			self.btns=[]
+
+			self.button_group = QtWidgets.QButtonGroup()
+
+			#moods[0].setChecked(True)
+			pos=0
+			for i in L:
+				# Add each radio button to the button layout
+				btn = QtWidgets.QRadioButton(i)
+				self.button_layout.addWidget(btn)
+				self.btns.append(btn)
+				if handler:
+					if handler.connected:
+						if handler.portname == i:
+							btn.setStyleSheet("color:green;")
+				if not L[i]: #Port in use
+					btn.setEnabled(False)
+
+				self.button_group.addButton(btn, pos)
+				pos+=1
+
+			# Set the layout of the group box to the button layout
+
+		#Print out the ID & text of the checked radio button
+		def finished(self):
+			if self.button_group.checkedId()!= -1:
+				self.done(QtWidgets.QDialog.Accepted)
+		def getSelection(self):
+			if self.button_group.checkedId()!= -1:
+				return self.button_group.checkedButton().text()
+			else:
+				return False
+
+
+
 	def showCredits(self):
 		if self.credwin == None:
 			self.credwin = helpWin(self, (self.tr('Credits'),('1.1','Credits')), self.lang)
@@ -482,8 +637,10 @@ class MainWindow(QMainWindow,QScreen):
 		if self.expName != 'scope':
 			explib = importlib.import_module('scope')
 			try:
-				if self.expWidget != None:
+				try:
 					self.expWidget.timer.stop()     # Stop the timer loop of current widget			
+				except:
+					pass
 				self.hwin = None
 				self.expWidget= None 			    # Let python delete it
 				w = explib.Expt(p) 
@@ -499,21 +656,29 @@ class MainWindow(QMainWindow,QScreen):
 		self.showHelp()
 	
 
-	def callExpt(self, e):
+	def callExpt(self, details):
 		"""
 		:parm: e lst with a title and a HTML file designation; when e[1]
 		is not a string, then it is an iterable with possible HTML file names,
 		and the last file name may also be a module name.
 		"""	
+		e = details
 		self.title=e[0] # record the title of the experiments, for snapshots
 		module_name =  e[1] if type(e[1]) is str else e[1][-1]
 		explib = importlib.import_module(module_name)
 		try:
-			if self.expWidget != None:
-				self.expWidget.timer.stop()	 # Stop the timer loop of current widget			
+			try:
+				self.expWidget.timer.stop()     # Stop the timer loop of current widget			
+			except:
+				pass
 			self.hwin = None
 			self.expWidget= None				 # Let python delete it
 			w = explib.Expt(p)
+			try:
+				w.callExptReference(self.callExpt, self.scope_help, self.runCode)
+			except:
+				pass
+
 			self.setWindowTitle(self.tr(e[0]))
 			self.setCentralWidget(w)
 			self.expWidget = w
@@ -521,8 +686,10 @@ class MainWindow(QMainWindow,QScreen):
 			self.hlpName = e[1]
 			self.title = e[0]
 			self.showHelp()
+			w.show()
+
 		except Exception as err:
-			logging.exception(err)	
+			print("Exception:", err)	
 			self.expName = ''
 			self.setWindowTitle(self.tr('Failed to load %s') %e[0])
 		return
@@ -677,7 +844,10 @@ class MainWindow(QMainWindow,QScreen):
 		try:
 			if not threadCalling:
 				#Timers cannot be stopped from another thread
-				self.expWidget.timer.stop()
+				try:
+					self.expWidget.timer.stop()     # Stop the timer loop of current widget			
+				except:
+					pass
 		except:
 			pass
 
@@ -707,10 +877,10 @@ class MainWindow(QMainWindow,QScreen):
 			d=datetime.now()
 			generator.setTitle(d.strftime("ExpEYES 17 Screenshot--%Y-%m-%d--%H-%M"))
 			generator.setDescription(self.title)
-			p = QtGui.QPainter()
-			p.begin(generator)
-			self.render(p)
-			p.end()
+			ptr = QtGui.QPainter()
+			ptr.begin(generator)
+			self.render(ptr)
+			ptr.end()
 			# fix the width of oscilloscope's traces, for SVG readers which
 			# do not honor the attribute 'vector-effect = "non-scaling-stroke"'
 			fixNonScalingStroke(path)
@@ -906,10 +1076,10 @@ You can customize the way they are used to build the path."""
 				d=datetime.now()
 				generator.setTitle(d.strftime("ExpEYES 17 Smaller Screenshot--%Y-%m-%d--%H-%M"))
 				generator.setDescription(self.title)
-				p = QtGui.QPainter()
-				p.begin(generator)
-				self.render(p)
-				p.end()
+				ptr = QtGui.QPainter()
+				ptr.begin(generator)
+				self.render(ptr)
+				ptr.end()
 				# fix the width of oscilloscope's traces, for SVG readers which
 				# do not honor the attribute 'vector-effect = "non-scaling-stroke"'
 				fixNonScalingStroke(path)
@@ -931,13 +1101,13 @@ You can customize the way they are used to build the path."""
 		
 		return
 
-	def reconnect(self):
+	def reconnect(self, **kwargs):
 		global p,eyes
 		try:
 			p.H.disconnect()
 		except:
 			pass
-		p=eyes.open()
+		p=eyes.open(**kwargs)
 		if self.expWidget is None:
 			explib = importlib.import_module('scope')
 			self.expWidget = explib.Expt(p) 
@@ -967,7 +1137,7 @@ You can customize the way they are used to build the path."""
 		self.app.installTranslator(self.tr_eyes)
 		self.tr_qt=QTranslator()
 		self.tr_qt.load("qt_"+lang,
-					QLibraryInfo.path(QLibraryInfo.LibraryPath(10))) # 10 = QLibraryInfo.TranslationsPath
+			QLibraryInfo.location(QLibraryInfo.TranslationsPath))
 		self.app.installTranslator(self.tr_qt)
 		self.uncheckHelpBox.emit()
 
@@ -976,7 +1146,7 @@ def run():
 	global app,p,eyes
 	import eyes17.eyes as eyes
 	p = eyes.open()
-	if p != None: 
+	if p.H.connected:
 		p.set_sine(1000)
 		p.set_sqr1(-1)
 		p.set_pv1(0)
@@ -992,11 +1162,11 @@ def run():
 	app.installTranslator(t)
 	t1=QTranslator()
 	t1.load("qt_"+lang,
-		QLibraryInfo.path(QLibraryInfo.LibraryPath(10))) # 10 = QLibraryInfo.TranslationsPath
+		QLibraryInfo.location(QLibraryInfo.TranslationsPath))
 	app.installTranslator(t1)
 
 	mw = MainWindow(lang, app, t, t1)
-	sys.exit(app.exec())
+	sys.exit(app.exec_())
 
 if __name__ == '__main__':
 	run()
