@@ -5,4 +5,5 @@ fd = serial.Serial('/dev/ttyS0', 38400, stopbits=1, \
 while 1:
   c = raw_input('Enter a character : ')
   fd.write(c)	
-  print 'Receiced ', fd.read()
\ No newline at end of file
+  print ('Receiced ', fd.read())
+
